%convex_mirror
% Determines the virtual image of an object in a spherical convex mirror.
%
% LAST UPDATED by Andy French May 2017

function convex_mirror
global d

%Radius of mirror /m
d.R = 0.5;

%Width of object (this scales the image)
w = 0.5;

%Object centre
x0 = d.R + w/2 + 0.3;
y0 = 0.0;

%Movement on keypress
d.delta = 0.05;

%Fontsize
fsize = 18;

%

%Import bitmap image of object
d.I = imread('Maxwell.jpg');
d.I = double(flipdim(d.I,1))/255;
dim = size(d.I);
Iwidth = dim(2);
Iheight = dim(1);

%Determine x,y coordinates of pixel locations
d.x = linspace(0,w,Iwidth) - w/2 + x0;
d.y = linspace(0,w*Iheight/Iwidth,Iheight) - 0.5*w*Iheight/Iwidth + y0 ;
[d.x,d.y] = meshgrid(d.x,d.y);

%Detrmine most left x coordinate of object
d.xmin = x0-w/2;

%Determine largest and smallest y coordinates of object
d.ymax = 0.5*w*Iheight/Iwidth + y0;
d.ymin = -0.5*w*Iheight/Iwidth + y0;

%Create figure and plot object
d.fig = figure( 'KeyPressFcn',@keypress );
d.Obj = surf(d.x,d.y,-ones(Iheight,Iwidth),d.I);
shading interp
view(2);
hold on;
axis equal
xlabel('x','fontsize',fsize);
ylabel('y','fontsize',fsize);
title('Reflection in a convex mirror','fontsize',fsize);
set(gca,'fontsize',fsize);
xlim([-d.delta,x0+w]);
ylim([-d.delta-d.R,+d.delta+d.R]);
axis manual
box on

%Determine mirror
t = linspace(0,pi,500);
xm = d.R*sin(t);
ym = d.R*cos(t);
plot(xm,ym,'b')

%Plot mirror centre of curvature and approximate focus
plot(0,0,'b*','markersize',fsize);

%Define a particular location on image to draw rays from
d.a = x0;
d.b = 0.5*w*Iheight/Iwidth + y0;

%Find intersection of horizontal ray with mirror x coordinate
d.xm = sqrt( d.R^2 - d.b^2 );

%Determine transformation in mirror
[d.xx,d.yy] = mirror( d.x,d.y,d.R );
[d.aa,d.bb] = mirror( d.a,d.b,d.R );

%Plot transformed image
d.Imag = surf(d.xx,d.yy,-ones(Iheight,Iwidth),d.I);
shading interp

%Overlay rays from (d.a,d.b) to (d.aa,d.bb) via mirror
xlimits = get(gca,'xlim');
theta = atan( d.b/sqrt( d.R^2 -d.b^2 ) );
if 2*theta > pi/2
    d.ray1 = plot( [d.a,d.xm,xlimits(1)],[d.b,d.b,d.b + (xlimits(1) - d.xm)*(d.b-d.bb)/(d.xm-d.aa)],'r-' );
else
    d.ray1 = plot( [d.a,d.xm,xlimits(2)],[d.b,d.b,d.b + (xlimits(2) - d.xm)*(d.b-d.bb)/(d.xm-d.aa)],'r-' );
end
d.ray2 = plot( [d.aa,d.xm],[d.bb,d.b],'r--' );
d.ray3 = plot( [d.R/sqrt(1+(d.b/d.a)^2),d.a],[(d.b/d.a)*d.R/sqrt(1+(d.b/d.a)^2),d.b],'r-' );
d.ray4 = plot( [0,d.R/sqrt(1+(d.b/d.a)^2)],[0,(d.b/d.a)*d.R/sqrt(1+(d.b/d.a)^2)],'r--' );
d.ray5 = plot( [0,xlimits(2)],[0,xlimits(2)*d.b/d.xm],'b--' );
d.ab = plot( d.a,d.b,'r*' );
d.aabb = plot( d.aa,d.bb,'g*' );

%%

%Function which executes when a key is pressed
function keypress(fig,evnt)
global d

if strcmp(get(fig,'currentkey'),'uparrow')==1
    d.ymax = d.ymax + d.delta;
    if ( ( (d.xmin^2 + d.ymax^2) < d.R^2 ) || ( (d.xmin^2 + d.ymin^2) < d.R^2 ) ) || ( d.ymax > d.R )
        d.ymax =  d.ymax - d.delta;
    else
        d.y = d.y + d.delta;
        d.b = d.b + d.delta;
        d.ymin = d.ymin + d.delta;
        update
    end
elseif strcmp(get(fig,'currentkey'),'downarrow')==1
    d.ymin = d.ymin - d.delta;
    if ( ( (d.xmin^2 + d.ymax^2) < d.R^2 ) || ( (d.xmin^2 + d.ymin^2) < d.R^2 ) ) || ( d.ymin < -d.R )
        d.ymin = d.ymin + d.delta;
    else
        d.y = d.y - d.delta;
        d.b = d.b - d.delta;
        d.ymax = d.ymax - d.delta;
        update
    end
elseif strcmp(get(fig,'currentkey'),'leftarrow')==1
    d.xmin = d.xmin - d.delta;
    if ( (d.xmin^2 + d.ymax^2) < d.R^2 ) || ( (d.xmin^2 + d.ymin^2) < d.R^2 )
        d.xmin = d.xmin + d.delta;
    else
        d.x = d.x - d.delta;
        d.a = d.a - d.delta;
        update
    end
elseif strcmp(get(fig,'currentkey'),'rightarrow')==1
    d.x = d.x + d.delta;
    d.a = d.a + d.delta;
    d.xmin = d.xmin + d.delta;
    update
elseif strcmp(get(fig,'currentkey'),'p')==1
    print( d.fig, 'convex_mirror.png','-dpng','-r300' );
end

%%

%Update screen
function update
global d

%Determine transformation in mirror
[d.xx,d.yy] = mirror( d.x,d.y,d.R );
[d.aa,d.bb] = mirror( d.a,d.b,d.R );

%Update object and image
xlimits = get(gca,'xlim');
set( d.Obj,'xdata',d.x,'ydata',d.y );
set( d.Imag,'xdata',d.xx,'ydata',d.yy );
d.xm = sqrt( d.R^2 - d.b^2 );
theta = atan( d.b/sqrt( d.R^2 -d.b^2 ) );
if 2*theta > pi/2
    set( d.ray1,'xdata',[d.a,d.xm,xlimits(1)],'ydata',[d.b,d.b,d.b + (xlimits(1) - d.xm)*(d.b-d.bb)/(d.xm-d.aa)] );
else
    set( d.ray1,'xdata',[d.a,d.xm,xlimits(2)],'ydata',[d.b,d.b,d.b + (xlimits(2) - d.xm)*(d.b-d.bb)/(d.xm-d.aa)] );
end
set( d.ray2,'xdata',[d.aa,d.xm],'ydata',[d.bb,d.b] );
set( d.ray3,'xdata',[d.R/sqrt(1+(d.b/d.a)^2),d.a],'ydata',[(d.b/d.a)*d.R/sqrt(1+(d.b/d.a)^2),d.b] );
set( d.ray4,'xdata',[0,d.R/sqrt(1+(d.b/d.a)^2)],'ydata',[0,(d.b/d.a)*d.R/sqrt(1+(d.b/d.a)^2)] );
set( d.ray5,'xdata',[0,xlimits(2)],'ydata',[0,xlimits(2)*d.b/d.xm] );
set(d.ab,'xdata',d.a,'ydata',d.b);
set(d.aabb,'xdata',d.aa,'ydata',d.bb);
drawnow;

%%

%Transformation in mirror
function [xx,yy] = mirror( x,y,R )
theta = atan( y./sqrt( -y.^2 + R^2 ) );
m = tan(2*theta);
xx = ( m.*sqrt( -y.^2 + R^2 ) - y)./( m - y./x);
yy = y.*xx./x;
bad = ( ( yy.^2 + xx.^2 ) > R^2 ) |  ( ( -y.^2 + R^2 ) < 0 ) |  ( ( y.^2 + x.^2 ) < R^2 );
xx(bad) = NaN;
yy(bad) = NaN;

%End of code