%diverging_thin_lens
% Determines the virtual image of an object in a thin diverging lens. (i.e.
% a bi-concave lens formed by two spherical sections. Assume spacing is
% negligible).
%
% LAST UPDATED by Andy French May 2017

function diverging_thin_lens
global d

%Height of lens
d.H = 1;

%Radius of curvature of lenses (merely for visual purposes - we don't make
%use of the Lensmaker's formula in this simulation)
Rc = 5;

%Width of lens (merely for visual purposes - we don't make
%use of the Lensmaker's formula in this simulation)
lw = 0.1;

%Focal length of lens
d.f = 0.6;

%Object centre
x0 = 1;
y0 = 0.0;

%Width of object (this scales the image)
w = 0.5;

%Movement on keypress
d.delta = 0.02;

%Fontsize
fsize = 18;

%

%Import bitmap image of object
d.I = imread('Maxwell.jpg');
d.I = double(flipdim(d.I,1))/255;
dim = size(d.I);
Iwidth = dim(2);
Iheight = dim(1);

%Determine x,y coordinates of pixel locations
d.x = linspace(0,w,Iwidth) - w/2 + x0;
d.y = linspace(0,w*Iheight/Iwidth,Iheight) - 0.5*w*Iheight/Iwidth + y0 ;
[d.x,d.y] = meshgrid(d.x,d.y);

%Detrmine most left x coordinate of object
d.xmin = x0-w/2;

%Determine largest and smallest y coordinates of object
d.ymax = 0.5*w*Iheight/Iwidth + y0;
d.ymin = -0.5*w*Iheight/Iwidth + y0;

%Create figure and plot object
d.fig = figure( 'KeyPressFcn',@keypress );
d.Obj = surf(d.x,d.y,-ones(Iheight,Iwidth),d.I);
shading interp
view(2);
hold on;
axis equal
xlabel('x','fontsize',fsize);
ylabel('y','fontsize',fsize);
title('Virtual image of object in a diverging lens','fontsize',fsize);
set(gca,'fontsize',fsize);
xlim([-d.f,2*d.f]);
ylim([-d.delta-d.H,+d.delta+d.H]);
axis manual
box on

%Determin x coordinate of centre of curvature for lenses
xR = Rc + lw/2;
xL = - lw/2 - Rc;

%Determine lens arcs
tmax = asin( d.H/Rc );
t = linspace(-tmax,tmax,500);
xl = xL + Rc*cos(t);
yl = Rc*sin(t);
xr = xR - Rc*cos(t);
yr = Rc*sin(t);
plot([xl,fliplr(xr),xl(1)],[yl,fliplr(yr),yl(1)],'b')

%Plot lens foci
plot(d.f,0,'b*','markersize',fsize);

%Define a particular location on image to draw rays from
d.a = x0;
d.b = 0.5*w*Iheight/Iwidth + y0;

%Determine image location due to action of lens
[d.xx,d.yy] = lens( d.x,d.y,d.f );
[d.aa,d.bb] = lens( d.a,d.b,d.f );

%Plot transformed image
d.Imag = surf(d.xx,d.yy,-ones(Iheight,Iwidth),d.I);
shading interp

%Overlay rays from (d.a,d.b) to (d.aa,d.bb) via mirror
xlimits = get(gca,'xlim');
d.ray1 = plot( [d.a,xlimits(1)],[d.b,d.b*xlimits(1)/d.a],'r-' );
d.ray2 = plot( [d.a,0,xlimits(1)],[d.b,d.b,d.b - xlimits(1)*d.b/d.f],'r-' );
d.ray3 = plot( [0,d.f],[d.b,0],'r--' );
d.ab = plot( d.a,d.b,'r*' );
d.aabb = plot( d.aa,d.bb,'g*' );

%%

%Function which executes when a key is pressed
function keypress(fig,evnt)
global d

if strcmp(get(fig,'currentkey'),'uparrow')==1
    d.ymax = d.ymax + d.delta;
    %Check that object is not beyond top of lens
    if d.ymax > d.H
        d.ymax = d.ymax - d.delta;
    else
        d.y = d.y + d.delta;
        d.b = d.b + d.delta;
        d.ymin = d.ymin + d.delta;
        update
    end
elseif strcmp(get(fig,'currentkey'),'downarrow')==1
    d.ymin = d.ymin - d.delta;
    %Check that object is not below bottom of lens
    if d.ymin < -d.H
        d.ymin = d.ymin + d.delta;
    else
        d.y = d.y - d.delta;
        d.b = d.b - d.delta;
        d.ymax = d.ymax - d.delta;
        update
    end
elseif strcmp(get(fig,'currentkey'),'leftarrow')==1    
    d.xmin = d.xmin - d.delta;
    %Check that object is not within focal length
    if d.xmin < d.f
        d.xmin = d.xmin + d.delta;
    else
        d.x = d.x - d.delta;
        d.a = d.a - d.delta;
        update
    end
elseif strcmp(get(fig,'currentkey'),'rightarrow')==1
    d.x = d.x + d.delta;
    d.a = d.a + d.delta;
    d.xmin = d.xmin + d.delta;
    update
elseif strcmp(get(fig,'currentkey'),'p')==1
    print( d.fig, 'diverging_lens.png','-dpng','-r300' );
end

%%

%Update screen
function update
global d

%Determine transformation in mirror
[d.xx,d.yy] = lens( d.x,d.y,d.f );
[d.aa,d.bb] = lens( d.a,d.b,d.f );

%Update object and image
xlimits = get(gca,'xlim');
set( d.Obj,'xdata',d.x,'ydata',d.y );
set( d.Imag,'xdata',d.xx,'ydata',d.yy );
set( d.ray1,'xdata',[d.a,xlimits(1)],'ydata',[d.b,d.b*xlimits(1)/d.a]);
set( d.ray2,'xdata',[d.a,0,xlimits(1)],'ydata',[d.b,d.b,d.b - xlimits(1)*d.b/d.f]);
set( d.ray3,'xdata',[0,d.f],'ydata',[d.b,0]);
set(d.ab,'xdata',d.a,'ydata',d.b);
set(d.aabb,'xdata',d.aa,'ydata',d.bb);
drawnow;

%%

% Lens virtual image position 
function [xx,yy] = lens( x,y,f )
xx = ( ( 1./x + 1/f).^-1 );
yy = y.*xx ./x;

%End of code